@@ -87,6 +87,33 @@ static int records_put_opt_parse_cb(void * const context_,
     return 0;
 }
 
+typedef struct RecordsOptParseCBContext_ {
+    _Bool with_links;
+} RecordsOptParseCBContext;
+
+static int records_opt_parse_cb(void * const context_,
+                                const BinVal *key, const BinVal *value)
+{
+    RecordsOptParseCBContext * context = context_;
+    char *svalue = value->val;    
+    
+    skip_spaces((const char * *) &svalue);
+    if (*svalue == 0) {
+        return 0;
+    }
+    if (BINVAL_IS_EQUAL_TO_CONST_STRING(key, "links")) {
+        char *endptr;
+        _Bool with_links = (strtol(svalue, &endptr, 10) > 0);
+        if (endptr == NULL || endptr == svalue) {
+            return -1;
+        }
+        context->with_links = with_links;
+                
+        return 0;
+    }
+    return 0;
+}
+
 int handle_domain_records(struct evhttp_request * const req,
                           HttpHandlerContext * const context,
                           char *uri, char *opts, _Bool * const write_to_log,
@@ -106,6 +133,7 @@ int handle_domain_records(struct evhttp_request * const req,
         }
         *sep = 0;
         if ((layer_name = new_key_from_c_string(uri)) == NULL) {
+            *sep = '/';
             return HTTP_SERVUNAVAIL;
         }
         *sep++ = '/';
@@ -115,17 +143,25 @@ int handle_domain_records(struct evhttp_request * const req,
         }
         if ((key = new_key_from_c_string(sep)) == NULL) {
             release_key(layer_name);
-            
             return HTTP_SERVUNAVAIL;
         }
+        RecordsOptParseCBContext cb_context = {
+            .with_links = 0
+        };
+        if (opts != NULL &&
+            query_parse(opts, records_opt_parse_cb, &cb_context) != 0) {
+            release_key(layer_name);
+            return HTTP_BADREQUEST;
+        }
         *get_op = (RecordsGetOp) {
             .type = OP_TYPE_RECORDS_GET,
             .req = req,
             .fake_req = fake_req,
             .op_tid = ++context->op_tid,
             .layer_name = layer_name,
-            .key = key
-        };
+            .key = key,
+            .with_links = cb_context.with_links
+        };        
         pthread_mutex_lock(&context->mtx_cqueue);
         if (push_cqueue(context->cqueue, get_op) != 0) {
             pthread_mutex_unlock(&context->mtx_cqueue);            
@@ -158,8 +194,7 @@ int handle_domain_records(struct evhttp_request * const req,
             return HTTP_NOTFOUND;
         }
         if ((key = new_key_from_c_string(sep)) == NULL) {
-            release_key(layer_name);
-            
+            release_key(layer_name);            
             return HTTP_SERVUNAVAIL;
         }
         evbuffer_add(evhttp_request_get_input_buffer(req), "", (size_t) 1U);
@@ -226,8 +261,7 @@ int handle_domain_records(struct evhttp_request * const req,
             return HTTP_NOTFOUND;
         }
         if ((key = new_key_from_c_string(sep)) == NULL) {
-            release_key(layer_name);
-            
+            release_key(layer_name);            
             return HTTP_SERVUNAVAIL;
         }
         *delete_op = (RecordsDeleteOp) {
@@ -405,11 +439,31 @@ int handle_op_records_put(RecordsPutOp * const put_op,
         free_slip_map(&put_op->special_properties);        
         return HTTP_NOTFOUND;
     }
-    assert(status >= 0);
-    if (status == 0 && put_op->position_set != 0 && key_node->slot != NULL) {
-        remove_entry_from_key_node(pan_db, key_node, 0);
-        assert(key_node->slot != NULL);
-        key_node->slot = NULL;
+    assert(status > 0);
+    const Rectangle2D * const qbounds = &pan_db->qbounds;
+    if (put_op->position_set != 0 &&
+        !(put_op->position.latitude >= qbounds->edge0.latitude &&
+          put_op->position.longitude >= qbounds->edge0.longitude &&
+          put_op->position.latitude < qbounds->edge1.latitude &&
+          put_op->position.longitude < qbounds->edge1.longitude)) {
+        put_op->position_set = 0;
+    }
+    if (status > 0 && put_op->position_set != 0 && key_node->slot != NULL) {
+#if PROJECTION
+        const Position2D * const previous_position =
+            &key_node->slot->real_position;
+#else
+        const Position2D * const previous_position =
+            &key_node->slot->position;
+#endif
+        if (previous_position->latitude == put_op->position.latitude &&
+            previous_position->longitude == put_op->position.longitude) {
+            put_op->position_set = 0;
+        } else {
+            remove_entry_from_key_node(pan_db, key_node, 0);
+            assert(key_node->slot != NULL);
+            key_node->slot = NULL;
+        }
     }
     if (put_op->position_set != 0) {
         Slot slot;
@@ -532,7 +586,7 @@ int handle_op_records_get(RecordsGetOp * const get_op,
         assert(status <= 0);
         return HTTP_NOTFOUND;
     }
-    assert(status >= 0);
+    assert(status > 0);
     if (get_op->fake_req != 0) {
         return 0;
     }
@@ -554,8 +608,7 @@ int handle_op_records_get(RecordsGetOp * const get_op,
         return HTTP_SERVUNAVAIL;
     }
     get_op_reply->json_gen = json_gen;
-    key_node_to_json(key_node, json_gen, 1);
-    
+    key_node_to_json(key_node, json_gen, pan_db, 1, get_op->with_links);
     send_op_reply(context, op_reply);
     
     return 0;
@@ -584,7 +637,7 @@ int handle_op_records_delete(RecordsDeleteOp * const delete_op,
         assert(status <= 0);
         return HTTP_NOTFOUND;
     }
-    assert(status >= 0);
+    assert(status > 0);
     if (key_node->slot != NULL) {
         if (remove_entry_from_key_node(pan_db, key_node, 0) != 0) {
             return HTTP_SERVUNAVAIL;
