@@ -5,9 +5,12 @@
 #include "domain_layers.h"
 #include "domain_records.h"
 #include "domain_search.h"
+#include "public.h"
 #include "handle_consumer_ops.h"
 #include "expirables.h"
 #include "db_log.h"
+#include "replication_master.h"
+#include "replication_slave.h"
 
 static struct event_base *event_base;
 
@@ -127,6 +130,14 @@ stab:
 #endif
             ret = handle_op_search_in_rect(&op.search_in_rect_op, context);
             pthread_rwlock_unlock(&context->rwlock_layers);
+        } else if (op.bare_op.type == OP_TYPE_SEARCH_IN_KEYS) {
+#if AUTOMATICALLY_CREATE_LAYERS
+            pthread_rwlock_wrlock(&context->rwlock_layers);
+#else
+            pthread_rwlock_rdlock(&context->rwlock_layers);
+#endif
+            ret = handle_op_search_in_keys(&op.search_in_keys_op, context);
+            pthread_rwlock_unlock(&context->rwlock_layers);
         } else {
             assert(0);
         }
@@ -154,7 +165,8 @@ stab:
                             (unsigned int) sizeof "error" - (size_t) 1U);
             yajl_gen_string(json_gen, (const unsigned char *) "error",
                             (unsigned int) sizeof "error" - (size_t) 1U);
-            send_op_reply(context, op_reply);                
+            send_op_reply(context, op_reply);
+            yajl_gen_free(json_gen);
         }
     } else {
         pthread_cond_wait(&context->cond_cqueue, &context->mtx_cqueue);
@@ -166,45 +178,50 @@ stab:
 static void *worker_thread(void *context_)
 {
     HttpHandlerContext * const context = context_;
-    logfile(context, LOG_INFO, "Starting worker thread: [%p]",
-            (void *) pthread_self());
+    uint32_t thr_id;
     
+    evutil_secure_rng_get_bytes(&thr_id, sizeof thr_id);
+    logfile(context, LOG_INFO,
+            "Starting worker thread: [%" PRIu32 "]", thr_id);
+
     while (worker_do_work(context) == 0);
     
-    logfile(context, LOG_INFO, "Exited worker thread: [%p]",
-            (void *) pthread_self());
+    logfile(context, LOG_INFO, "Exited worker thread: [%" PRIu32 "]", thr_id);
     
     return NULL;
 }
 
-static int process_request(HttpHandlerContext *context,
-                           struct evhttp_request * const req,
-                           const _Bool fake_req)
+static char *extract_opts(char * const decoded_uri)
 {
-    const char *uri;    
-    char *decoded_uri;
-    size_t decoded_uri_len;
-    const char *ext = ".json";
-    const size_t ext_len = strlen(ext);
-    char *domain;
-    char *pnt;
-    
-    uri = evhttp_request_get_uri(req);
-    if (strncmp(uri, context->encoded_base_uri,
-                context->encoded_base_uri_len) != 0) {
-        if (fake_req == 0) {
-            evhttp_send_error(req, HTTP_NOTFOUND, "Not Found");
-        }
-        return -1;
-    }
-    decoded_uri = evhttp_decode_uri(uri + context->encoded_base_uri_len);
     char * opts;
+    
     if ((opts = strchr(decoded_uri, '?')) != NULL) {
         *opts = 0;
         if (*++opts == 0) {
             opts = NULL;
         }
     }
+    return opts;
+}
+
+static int process_api_request(HttpHandlerContext * const context,
+                               const char * const uri,
+                               struct evhttp_request * const req,
+                               const _Bool fake_req)
+{
+    char *decoded_uri;
+    size_t decoded_uri_len;
+    const char *ext = ".json";
+    const size_t ext_len = strlen(ext);
+    char *domain;
+    char *pnt;
+
+    if (fake_req == 0) {
+        evhttp_add_header(req->output_headers, "Content-Type",
+                          "application/json; charset=UTF-8");    
+    }
+    decoded_uri = evhttp_decode_uri(uri + context->encoded_api_base_uri_len);
+    char * const opts = extract_opts(decoded_uri);
     decoded_uri_len = strlen(decoded_uri);
     if (decoded_uri_len <= ext_len || decoded_uri_len > MAX_URI_LEN ||
         strcasecmp(decoded_uri + decoded_uri_len - ext_len, ext) != 0) {
@@ -251,13 +268,46 @@ static int process_request(HttpHandlerContext *context,
         if (fake_req == 0) {        
             evhttp_send_error(req, ret, "Error");
         }
-        return- 1;
+        return -1;
     }
     if (fake_req == 0 && write_to_log != 0) {
-        add_to_db_log(context->now, req->type, uri,
-                      evhttp_request_get_input_buffer(req));
+        add_to_db_log(context, req->type, uri,
+                      evhttp_request_get_input_buffer(req), 1);
     }
-    return 0;
+    return 0;    
+}
+
+static int process_public_request(HttpHandlerContext * const context,
+                                  const char * const uri,
+                                  struct evhttp_request * const req)
+{
+    char *decoded_uri;
+    decoded_uri = evhttp_decode_uri(uri + context->encoded_public_base_uri_len);
+    char * const opts = extract_opts(decoded_uri);
+    const int ret = handle_public_request(context, decoded_uri, opts, req);
+    free(decoded_uri);
+    
+    return ret;
+}
+
+static int process_request(HttpHandlerContext * const context,
+                           struct evhttp_request * const req,
+                           const _Bool fake_req)
+{
+    const char * const uri = evhttp_request_get_uri(req);
+    if (strncmp(uri, context->encoded_api_base_uri,
+                context->encoded_api_base_uri_len) == 0) {
+        return process_api_request(context, uri, req, fake_req);
+    }
+    if (strncmp(uri, context->encoded_public_base_uri,
+                context->encoded_public_base_uri_len) == 0) {
+        assert(fake_req == 0);
+        return process_public_request(context, uri, req);
+    }
+    if (fake_req == 0) {
+        evhttp_send_error(req, HTTP_NOTFOUND, "Not Found");
+    }
+    return -1;
 }
 
 static void http_dispatcher_cb(struct evhttp_request * const req,
@@ -268,8 +318,6 @@ static void http_dispatcher_cb(struct evhttp_request * const req,
     evhttp_connection_set_closecb(evhttp_request_get_connection(req),
                                   http_close_cb, req);
     evhttp_add_header(req->output_headers, "X-Server", SERVER_NAME);
-    evhttp_add_header(req->output_headers, "Content-Type",
-                      "application/json; charset=UTF-8");
 #ifdef NO_KEEPALIVE
     evhttp_add_header(req->output_headers, "Connection", "close");
 #endif
@@ -282,7 +330,9 @@ static void http_dispatcher_cb(struct evhttp_request * const req,
 
 int fake_request(HttpHandlerContext * const context,
                  const int verb, char * const uri,
-                 const char *body, const size_t body_len)
+                 const char *body, const size_t body_len,
+                 const _Bool in_main_thread,
+                 const _Bool add_to_journal)
 {
     struct evhttp_request req;
     
@@ -291,8 +341,16 @@ int fake_request(HttpHandlerContext * const context,
     req.uri = uri;
     req.input_buffer = evbuffer_new();
     evbuffer_add(req.input_buffer, body, body_len);
-    process_request(context, &req, 1);
+    if (add_to_journal != 0) {
+        add_to_db_log(context, verb, uri, req.input_buffer, 1);
+    }
+    while (process_request(context, &req, 1) != 0) {
+        usleep(1000000U / 25U);
+    }
     evbuffer_free(req.input_buffer);
+    if (in_main_thread == 0) {
+        return 0;
+    }
     worker_do_work(context);
     
     return 0;
@@ -344,8 +402,9 @@ static RETSIGTYPE sigchld_cb(const int sig)
         }
         assert(app_context.db_log.journal_rewrite_process != (pid_t) -1);
         assert(pid == app_context.db_log.journal_rewrite_process);
-        if (pid == app_context.db_log.journal_rewrite_process) {
-            system_rewrite_after_fork_cb();       
+        if (pid == app_context.db_log.journal_rewrite_process &&
+            app_context.http_handler_context != NULL) {
+            system_rewrite_after_fork_cb(app_context.http_handler_context);
         }
     }        
 }
@@ -506,18 +565,21 @@ int http_server(void)
         .thr_workers = NULL,
         .event_base = NULL,
         .op_tid = (OpTID) 0U,
-        .encoded_base_uri = NULL,
+        .encoded_api_base_uri = NULL,
+        .encoded_public_base_uri = NULL,
         .cqueue = NULL,
         .publisher_bev = NULL,
         .consumer_bev = NULL,
         .nb_layers = (size_t) 0U,
         .log_file_name = NULL,
-        .log_fd = -1
+        .log_fd = -1,
+        .rm_context = NULL,
+        .rs_context = NULL
     };
     if (time(&http_handler_context.now) == (time_t) -1) {
         return -1;
     }
-    http_handler_context.log_file_name = app_context.log_file_name;    
+    http_handler_context.log_file_name = app_context.log_file_name;
     if (open_log_file(&http_handler_context) != 0) {
         return -1;
     }
@@ -543,9 +605,12 @@ int http_server(void)
     pthread_mutex_init(&http_handler_context.mtx_cqueue, NULL);
     pthread_cond_init(&http_handler_context.cond_cqueue, NULL);
     pthread_rwlock_init(&http_handler_context.rwlock_layers, NULL);
-    http_handler_context.encoded_base_uri = ENCODED_BASE_URI;
-    http_handler_context.encoded_base_uri_len =
-        strlen(http_handler_context.encoded_base_uri);
+    http_handler_context.encoded_api_base_uri = ENCODED_API_BASE_URI;
+    http_handler_context.encoded_api_base_uri_len =
+        strlen(http_handler_context.encoded_api_base_uri);
+    http_handler_context.encoded_public_base_uri = ENCODED_PUBLIC_BASE_URI;
+    http_handler_context.encoded_public_base_uri_len =
+        strlen(http_handler_context.encoded_public_base_uri);
     evthread_use_pthreads();
     event_base = event_base_new();
     http_handler_context.event_base = event_base;
@@ -579,10 +644,26 @@ int http_server(void)
         };
         evtimer_add(&http_handler_context.ev_flush_log_db, &tv);
     }
-    replay_log(&http_handler_context);
+    app_context.http_handler_context = &http_handler_context;
+    if (app_context.replication_slave_ip != NULL &&
+        app_context.replication_slave_port != NULL) {
+        reset_log(&http_handler_context);
+        if (start_replication_slave(&http_handler_context,
+                                    app_context.replication_slave_ip,
+                                    app_context.replication_slave_port) != 0) {
+            logfile(&http_handler_context, LOG_ERR,
+                    "Unable to connect to the master");
+            goto bye;
+        }
+    } else {
+        replay_log(&http_handler_context);
+    }
     if (start_workers(&http_handler_context) != 0) {
         goto bye;
     }
+    start_replication_master(&http_handler_context,
+                             app_context.replication_master_ip,
+                             app_context.replication_master_port);
     evtimer_assign(&http_handler_context.ev_expiration_cron, event_base,
                    expiration_cron, &http_handler_context);
     struct timeval tv = {
@@ -599,6 +680,8 @@ int http_server(void)
     }
     evtimer_del(&http_handler_context.ev_expiration_cron);
 bye:
+    stop_replication_master(&http_handler_context);    
+    stop_replication_slave(&http_handler_context);        
     bufferevent_free(http_handler_context.publisher_bev);
     bufferevent_free(http_handler_context.consumer_bev);
     evhttp_free(event_http);
@@ -611,6 +694,7 @@ bye:
     free_slab(&http_handler_context.expirables_slab,
               free_expirables_slab_entry_cb);
     close_log_file(&http_handler_context);
+    app_context.http_handler_context = NULL;
     
     return 0;
 }
