@@ -56,9 +56,9 @@ ev_token_bucket_init(struct ev_token_bucket *bucket,
 		   leave "last_updated" as it is; the next update will add the
 		   appropriate amount of bandwidth to the bucket.
 		*/
-		if (bucket->read_limit > cfg->read_maximum)
+		if (bucket->read_limit > (ev_int64_t) cfg->read_maximum)
 			bucket->read_limit = cfg->read_maximum;
-		if (bucket->write_limit > cfg->write_maximum)
+		if (bucket->write_limit > (ev_int64_t) cfg->write_maximum)
 			bucket->write_limit = cfg->write_maximum;
 	} else {
 		bucket->read_limit = cfg->read_rate;
@@ -221,7 +221,7 @@ _bufferevent_get_rlim_max(struct bufferevent_private *bev, int is_write)
 	if (bev->rate_limiting->group) {
 		struct bufferevent_rate_limit_group *g =
 		    bev->rate_limiting->group;
-		ev_uint32_t share;
+		ev_int32_t share;
 		LOCK_GROUP(g);
 		if (GROUP_SUSPENDED(g)) {
 			/* We can get here if we failed to lock this
@@ -245,6 +245,8 @@ _bufferevent_get_rlim_max(struct bufferevent_private *bev, int is_write)
 		CLAMPTO(share);
 	}
 
+	if (max_so_far < 0)
+		max_so_far = 0;
 	return max_so_far;
 }
 
@@ -541,15 +543,19 @@ bufferevent_set_rate_limit(struct bufferevent *bev,
 	struct bufferevent_rate_limit *rlim;
 	struct timeval now;
 	ev_uint32_t tick;
+	int reinit = 0, suspended = 0;
 	/* XXX reference-count cfg */
 
 	BEV_LOCK(bev);
 
 	if (cfg == NULL) {
 		if (bevp->rate_limiting) {
-			bevp->rate_limiting->cfg = NULL;
+			rlim = bevp->rate_limiting;
+			rlim->cfg = NULL;
 			bufferevent_unsuspend_read(bev, BEV_SUSPEND_BW);
 			bufferevent_unsuspend_write(bev, BEV_SUSPEND_BW);
+			if (event_initialized(&rlim->refill_bucket_event))
+				event_del(&rlim->refill_bucket_event);
 		}
 		r = 0;
 		goto done;
@@ -559,29 +565,48 @@ bufferevent_set_rate_limit(struct bufferevent *bev,
 	tick = ev_token_bucket_get_tick(&now, cfg);
 
 	if (bevp->rate_limiting && bevp->rate_limiting->cfg == cfg) {
-		;
-	} else if (bevp->rate_limiting) {
-		bevp->rate_limiting->cfg = cfg;
-		ev_token_bucket_init(&bevp->rate_limiting->limit, cfg, tick, 1);
-		if (bevp->rate_limiting->limit.read_limit > 0)
-			bufferevent_unsuspend_read(bev, BEV_SUSPEND_BW);
-		else
-			bufferevent_suspend_read(bev, BEV_SUSPEND_BW);
-		if (bevp->rate_limiting->limit.write_limit > 0)
-			bufferevent_unsuspend_write(bev, BEV_SUSPEND_BW);
-		else
-			bufferevent_suspend_write(bev, BEV_SUSPEND_BW);
-	} else {
+		/* no-op */
+		r = 0;
+		goto done;
+	}
+	if (bevp->rate_limiting == NULL) {
 		rlim = mm_calloc(1, sizeof(struct bufferevent_rate_limit));
 		if (!rlim)
 			goto done;
-		rlim->cfg = cfg;
-		ev_token_bucket_init(&rlim->limit, cfg, tick, 0);
-		evtimer_assign(&rlim->refill_bucket_event, bev->ev_base,
-		    _bev_refill_callback, bevp);
 		bevp->rate_limiting = rlim;
+	} else {
+		rlim = bevp->rate_limiting;
 	}
+	reinit = rlim->cfg != NULL;
+
+	rlim->cfg = cfg;
+	ev_token_bucket_init(&rlim->limit, cfg, tick, reinit);
+
+	if (reinit) {
+		EVUTIL_ASSERT(event_initialized(&rlim->refill_bucket_event));
+		event_del(&rlim->refill_bucket_event);
+	}
+	evtimer_assign(&rlim->refill_bucket_event, bev->ev_base,
+	    _bev_refill_callback, bevp);
+
+	if (rlim->limit.read_limit > 0) {
+		bufferevent_unsuspend_read(bev, BEV_SUSPEND_BW);
+	} else {
+		bufferevent_suspend_read(bev, BEV_SUSPEND_BW);
+		suspended=1;
+	}
+	if (rlim->limit.write_limit > 0) {
+		bufferevent_unsuspend_write(bev, BEV_SUSPEND_BW);
+	} else {
+		bufferevent_suspend_write(bev, BEV_SUSPEND_BW);
+		suspended = 1;
+	}
+
+	if (suspended)
+		event_add(&rlim->refill_bucket_event, &cfg->tick_timeout);
+
 	r = 0;
+
 done:
 	BEV_UNLOCK(bev);
 	return r;
@@ -630,9 +655,9 @@ bufferevent_rate_limit_group_set_cfg(
 		&g->rate_limit_cfg.tick_timeout, &cfg->tick_timeout, ==);
 	memcpy(&g->rate_limit_cfg, cfg, sizeof(g->rate_limit_cfg));
 
-	if (g->rate_limit.read_limit > cfg->read_maximum)
+	if (g->rate_limit.read_limit > (ev_int32_t)cfg->read_maximum)
 		g->rate_limit.read_limit = cfg->read_maximum;
-	if (g->rate_limit.write_limit > cfg->write_maximum)
+	if (g->rate_limit.write_limit > (ev_int32_t)cfg->write_maximum)
 		g->rate_limit.write_limit = cfg->write_maximum;
 
 	if (!same_tick) {
@@ -644,6 +669,17 @@ bufferevent_rate_limit_group_set_cfg(
 	return 0;
 }
 
+int
+bufferevent_rate_limit_group_set_min_share(
+	struct bufferevent_rate_limit_group *g,
+	size_t share)
+{
+	if (share > EV_INT32_MAX)
+		return -1;
+
+	g->min_share = share;
+	return 0;
+}
 
 void
 bufferevent_rate_limit_group_free(struct bufferevent_rate_limit_group *g)
@@ -705,6 +741,13 @@ bufferevent_add_to_rate_limit_group(struct bufferevent *bev,
 
 int
 bufferevent_remove_from_rate_limit_group(struct bufferevent *bev)
+{
+	return bufferevent_remove_from_rate_limit_group_internal(bev, 1);
+}
+
+int
+bufferevent_remove_from_rate_limit_group_internal(struct bufferevent *bev,
+    int unsuspend)
 {
 	struct bufferevent_private *bevp =
 	    EVUTIL_UPCAST(bev, struct bufferevent_private, bev);
@@ -718,8 +761,10 @@ bufferevent_remove_from_rate_limit_group(struct bufferevent *bev)
 		TAILQ_REMOVE(&g->members, bevp, rate_limiting->next_in_group);
 		UNLOCK_GROUP(g);
 	}
-	bufferevent_unsuspend_read(bev, BEV_SUSPEND_BW_GROUP);
-	bufferevent_unsuspend_write(bev, BEV_SUSPEND_BW_GROUP);
+	if (unsuspend) {
+		bufferevent_unsuspend_read(bev, BEV_SUSPEND_BW_GROUP);
+		bufferevent_unsuspend_write(bev, BEV_SUSPEND_BW_GROUP);
+	}
 	BEV_UNLOCK(bev);
 	return 0;
 }
@@ -834,7 +879,8 @@ bufferevent_decrement_read_limit(struct bufferevent *bev, ev_ssize_t decr)
 			&bevp->rate_limiting->cfg->tick_timeout) < 0)
 			r = -1;
 	} else if (old_limit <= 0 && new_limit > 0) {
-		event_del(&bevp->rate_limiting->refill_bucket_event);
+		if (!(bevp->write_suspended & BEV_SUSPEND_BW))
+			event_del(&bevp->rate_limiting->refill_bucket_event);
 		bufferevent_unsuspend_read(bev, BEV_SUSPEND_BW);
 	}
 
@@ -862,7 +908,8 @@ bufferevent_decrement_write_limit(struct bufferevent *bev, ev_ssize_t decr)
 			&bevp->rate_limiting->cfg->tick_timeout) < 0)
 			r = -1;
 	} else if (old_limit <= 0 && new_limit > 0) {
-		event_del(&bevp->rate_limiting->refill_bucket_event);
+		if (!(bevp->read_suspended & BEV_SUSPEND_BW))
+			event_del(&bevp->rate_limiting->refill_bucket_event);
 		bufferevent_unsuspend_write(bev, BEV_SUSPEND_BW);
 	}
 
