@@ -24,7 +24,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
-#include "event-config.h"
+#include "event2/event-config.h"
 
 #ifdef WIN32
 #include <winsock2.h>
@@ -67,6 +67,7 @@
 #include "iocp-internal.h"
 #include "changelist-internal.h"
 #include "ht-internal.h"
+#include "util-internal.h"
 
 #ifdef _EVENT_HAVE_EVENT_PORTS
 extern const struct eventop evportops;
@@ -119,7 +120,6 @@ static const struct eventop *eventops[] = {
 /* Global state; deprecated */
 struct event_base *event_global_current_base_ = NULL;
 #define current_base event_global_current_base_
-extern struct event_base *evsig_base;
 
 /* Global state */
 
@@ -446,7 +446,7 @@ event_is_method_disabled(const char *name)
 
 	evutil_snprintf(environment, sizeof(environment), "EVENT_NO%s", name);
 	for (i = 8; environment[i] != '\0'; ++i)
-		environment[i] = toupper(environment[i]);
+		environment[i] = EVUTIL_TOUPPER(environment[i]);
 	/* Note that evutil_getenv() ignores the environment entirely if
 	 * we're setuid */
 	return (evutil_getenv(environment) != NULL);
@@ -470,7 +470,7 @@ static void
 notify_base_cbq_callback(struct deferred_cb_queue *cb, void *baseptr)
 {
 	struct event_base *base = baseptr;
-	if (!EVBASE_IN_THREAD(base))
+	if (EVBASE_NEED_NOTIFY(base))
 		evthread_notify_base(base);
 }
 
@@ -603,8 +603,7 @@ event_base_new_with_config(const struct event_config *cfg)
 		EVTHREAD_ALLOC_LOCK(base->th_base_lock,
 		    EVTHREAD_LOCKTYPE_RECURSIVE);
 		base->defer_queue.lock = base->th_base_lock;
-		EVTHREAD_ALLOC_LOCK(base->current_event_lock,
-		    EVTHREAD_LOCKTYPE_RECURSIVE);
+		EVTHREAD_ALLOC_COND(base->current_event_cond);
 		r = evthread_make_base_notifiable(base);
 		if (r<0) {
 			event_base_free(base);
@@ -615,19 +614,19 @@ event_base_new_with_config(const struct event_config *cfg)
 
 #ifdef WIN32
 	if (cfg && (cfg->flags & EVENT_BASE_FLAG_STARTUP_IOCP))
-		event_base_start_iocp(base);
+		event_base_start_iocp(base, cfg->n_cpus_hint);
 #endif
 
 	return (base);
 }
 
 int
-event_base_start_iocp(struct event_base *base)
+event_base_start_iocp(struct event_base *base, int n_cpus)
 {
 #ifdef WIN32
 	if (base->iocp)
 		return 0;
-	base->iocp = event_iocp_port_launch();
+	base->iocp = event_iocp_port_launch(n_cpus);
 	if (!base->iocp) {
 		event_warnx("%s: Couldn't launch IOCP", __func__);
 		return -1;
@@ -638,6 +637,20 @@ event_base_start_iocp(struct event_base *base)
 #endif
 }
 
+void
+event_base_stop_iocp(struct event_base *base)
+{
+#ifdef WIN32
+	int rv;
+
+	if (!base->iocp)
+		return;
+	rv = event_iocp_shutdown(base->iocp, -1);
+	EVUTIL_ASSERT(rv >= 0);
+	base->iocp = NULL;
+#endif
+}
+
 void
 event_base_free(struct event_base *base)
 {
@@ -654,6 +667,10 @@ event_base_free(struct event_base *base)
 	/* XXX(niels) - check for internal events first */
 	EVUTIL_ASSERT(base);
 
+#ifdef WIN32
+	event_base_stop_iocp(base);
+#endif
+
 	/* threading fds if we have them */
 	if (base->th_notify_fd[0] != -1) {
 		event_del(&base->th_notify);
@@ -730,8 +747,7 @@ event_base_free(struct event_base *base)
 	event_changelist_freemem(&base->changelist);
 
 	EVTHREAD_FREE_LOCK(base->th_base_lock, EVTHREAD_LOCKTYPE_RECURSIVE);
-	EVTHREAD_FREE_LOCK(base->current_event_lock,
-	    EVTHREAD_LOCKTYPE_RECURSIVE);
+	EVTHREAD_FREE_COND(base->current_event_cond);
 
 	mm_free(base);
 }
@@ -743,6 +759,7 @@ event_reinit(struct event_base *base)
 	const struct eventop *evsel;
 	int res = 0;
 	struct event *ev;
+	int was_notifiable = 0;
 
 	if (base == NULL) {
 		return -1;
@@ -751,9 +768,16 @@ event_reinit(struct event_base *base)
 
 	evsel = base->evsel;
 
+#if 0
+	/* Right now, reinit always takes effect, since even if the
+	   backend doesn't require it, the signal socketpair code does.
+
+	   XXX
+	 */
 	/* check if this event mechanism requires reinit */
 	if (!evsel->need_reinit)
 		goto done;
+#endif
 
 	/* prevent internal delete */
 	if (base->sig.ev_signal_added) {
@@ -766,6 +790,23 @@ event_reinit(struct event_base *base)
 			    EVLIST_ACTIVE);
 		base->sig.ev_signal_added = 0;
 	}
+	if (base->th_notify_fd[0] != -1) {
+		/* we cannot call event_del here because the base has
+		 * not been reinitialized yet. */
+		was_notifiable = 1;
+		event_queue_remove(base, &base->th_notify,
+		    EVLIST_INSERTED);
+		if (base->th_notify.ev_flags & EVLIST_ACTIVE)
+			event_queue_remove(base, &base->th_notify,
+			    EVLIST_ACTIVE);
+		base->sig.ev_signal_added = 0;
+		EVUTIL_CLOSESOCKET(base->th_notify_fd[0]);
+		if (base->th_notify_fd[1] != -1)
+			EVUTIL_CLOSESOCKET(base->th_notify_fd[1]);
+		base->th_notify_fd[0] = -1;
+		base->th_notify_fd[1] = -1;
+		event_debug_unassign(&base->th_notify);
+	}
 
 	if (base->evsel->dealloc != NULL)
 		base->evsel->dealloc(base);
@@ -791,6 +832,9 @@ event_reinit(struct event_base *base)
 		}
 	}
 
+	if (was_notifiable && res == 0)
+		res = evthread_make_base_notifiable(base);
+
 done:
 	EVBASE_RELEASE_LOCK(base, th_base_lock);
 	return (res);
@@ -897,6 +941,15 @@ event_config_require_features(struct event_config *cfg,
 	return (0);
 }
 
+int
+event_config_set_num_cpus_hint(struct event_config *cfg, int cpus)
+{
+	if (!cfg)
+		return (-1);
+	cfg->n_cpus_hint = cpus;
+	return (0);
+}
+
 int
 event_priority_init(int npriorities)
 {
@@ -941,7 +994,7 @@ static int
 event_haveevents(struct event_base *base)
 {
 	/* Caller must hold th_base_lock */
-	return (base->event_count > 0);
+	return (base->virtual_event_count > 0 || base->event_count > 0);
 }
 
 /* "closure" function called when processing active signal events */
@@ -957,6 +1010,8 @@ event_signal_closure(struct event_base *base, struct event *ev)
 	while (ncalls) {
 		ncalls--;
 		ev->ev_ncalls = ncalls;
+		if (ncalls == 0)
+			ev->ev_pncalls = NULL;
 		(*ev->ev_callback)((int)ev->ev_fd, ev->ev_res, ev->ev_arg);
 #if 0
 		/* XXXX we can't do this without a lock on the base. */
@@ -1220,9 +1275,10 @@ event_process_active_single_queue(struct event_base *base,
 			ev->ev_res & EV_WRITE ? "EV_WRITE " : " ",
 			ev->ev_callback));
 
+#ifndef _EVENT_DISABLE_THREAD_SUPPORT
 		base->current_event = ev;
-
-		EVBASE_ACQUIRE_LOCK(base, current_event_lock);
+		base->current_event_waiters = 0;
+#endif
 
 		switch (ev->ev_closure) {
 		case EV_CLOSURE_SIGNAL:
@@ -1239,9 +1295,14 @@ event_process_active_single_queue(struct event_base *base,
 			break;
 		}
 
-		EVBASE_RELEASE_LOCK(base, current_event_lock);
 		EVBASE_ACQUIRE_LOCK(base, th_base_lock);
+#ifndef _EVENT_DISABLE_THREAD_SUPPORT
 		base->current_event = NULL;
+		if (base->current_event_waiters) {
+			base->current_event_waiters = 0;
+			EVTHREAD_COND_BROADCAST(base->current_event_cond);
+		}
+#endif
 
 		if (base->event_break)
 			return -1;
@@ -1250,9 +1311,10 @@ event_process_active_single_queue(struct event_base *base,
 }
 
 /*
-   Process all the defered_cb entries in 'queue'.  If *breakptr becomes set to
-   1, stop.  Requires that we start out holding the lock on 'queue'; releases
-   the lock around 'queue' for each deferred_cb we process.
+   Process up to MAX_DEFERRED of the defered_cb entries in 'queue'.  If
+   *breakptr becomes set to 1, stop.  Requires that we start out holding
+   the lock on 'queue'; releases the lock around 'queue' for each deferred_cb
+   we process.
  */
 static int
 event_process_deferred_callbacks(struct deferred_cb_queue *queue, int *breakptr)
@@ -1260,6 +1322,7 @@ event_process_deferred_callbacks(struct deferred_cb_queue *queue, int *breakptr)
 	int count = 0;
 	struct deferred_cb *cb;
 
+#define MAX_DEFERRED 16
 	while ((cb = TAILQ_FIRST(&queue->deferred_cb_list))) {
 		cb->queued = 0;
 		TAILQ_REMOVE(&queue->deferred_cb_list, cb, cb_next);
@@ -1267,12 +1330,14 @@ event_process_deferred_callbacks(struct deferred_cb_queue *queue, int *breakptr)
 		UNLOCK_DEFERRED_QUEUE(queue);
 
 		cb->cb(cb, cb->arg);
-		++count;
-		if (*breakptr)
-			return -1;
 
 		LOCK_DEFERRED_QUEUE(queue);
+		if (*breakptr)
+			return -1;
+		if (++count == MAX_DEFERRED)
+			break;
 	}
+#undef MAX_DEFERRED
 	return count;
 }
 
@@ -1361,18 +1426,20 @@ event_loopbreak(void)
 int
 event_base_loopbreak(struct event_base *event_base)
 {
+	int r = 0;
 	if (event_base == NULL)
 		return (-1);
 
 	EVBASE_ACQUIRE_LOCK(event_base, th_base_lock);
 	event_base->event_break = 1;
-	EVBASE_RELEASE_LOCK(event_base, th_base_lock);
 
-	if (!EVBASE_IN_THREAD(event_base)) {
-		return evthread_notify_base(event_base);
+	if (EVBASE_NEED_NOTIFY(event_base)) {
+		r = evthread_notify_base(event_base);
 	} else {
-		return (0);
+		r = (0);
 	}
+	EVBASE_RELEASE_LOCK(event_base, th_base_lock);
+	return r;
 }
 
 int
@@ -1434,8 +1501,9 @@ event_base_loop(struct event_base *base, int flags)
 
 	clear_time_cache(base);
 
-	if (base->sig.ev_signal_added)
-		evsig_base = base;
+	if (base->sig.ev_signal_added && base->sig.ev_n_signals_added)
+		evsig_set_base(base);
+
 	done = 0;
 
 #ifndef _EVENT_DISABLE_THREAD_SUPPORT
@@ -1825,6 +1893,11 @@ event_add(struct event *ev, const struct timeval *tv)
 {
 	int res;
 
+	if (EVUTIL_FAILURE_CHECK(!ev->ev_base)) {
+		event_warnx("%s: event has no event_base set.", __func__);
+		return -1;
+	}
+
 	EVBASE_ACQUIRE_LOCK(ev->ev_base, th_base_lock);
 
 	res = event_add_internal(ev, tv, 0);
@@ -1849,7 +1922,7 @@ evthread_notify_base_default(struct event_base *base)
 #else
 	r = write(base->th_notify_fd[1], buf, 1);
 #endif
-	return (r < 0) ? -1 : 0;
+	return (r < 0 && errno != EAGAIN) ? -1 : 0;
 }
 
 #if defined(_EVENT_HAVE_EVENTFD) && defined(_EVENT_HAVE_SYS_EVENTFD_H)
@@ -1874,8 +1947,12 @@ evthread_notify_base_eventfd(struct event_base *base)
 static int
 evthread_notify_base(struct event_base *base)
 {
+	EVENT_BASE_ASSERT_LOCKED(base);
 	if (!base->th_notify_fn)
 		return -1;
+	if (base->is_notify_pending)
+		return 0;
+	base->is_notify_pending = 1;
 	return base->th_notify_fn(base);
 }
 
@@ -1895,8 +1972,9 @@ event_add_internal(struct event *ev, const struct timeval *tv,
 	_event_debug_assert_is_setup(ev);
 
 	event_debug((
-		 "event_add: event: %p, %s%s%scall %p",
+		 "event_add: event: %p (fd %d), %s%s%scall %p",
 		 ev,
+		 (int)ev->ev_fd,
 		 ev->ev_events & EV_READ ? "EV_READ " : " ",
 		 ev->ev_events & EV_WRITE ? "EV_WRITE " : " ",
 		 tv ? "EV_TIMEOUT " : " ",
@@ -1914,6 +1992,18 @@ event_add_internal(struct event *ev, const struct timeval *tv,
 			return (-1);  /* ENOMEM == errno */
 	}
 
+	/* If the main thread is currently executing a signal event's
+	 * callback, and we are not the main thread, then we want to wait
+	 * until the callback is done before we mess with the event, or else
+	 * we can race on ev_ncalls and ev_pncalls below. */
+#ifndef _EVENT_DISABLE_THREAD_SUPPORT
+	if (base->current_event == ev && (ev->ev_events & EV_SIGNAL)
+	    && !EVBASE_IN_THREAD(base)) {
+		++base->current_event_waiters;
+		EVTHREAD_COND_WAIT(base->current_event_cond, base->th_base_lock);
+	}
+#endif
+
 	if ((ev->ev_events & (EV_READ|EV_WRITE|EV_SIGNAL)) &&
 	    !(ev->ev_flags & (EVLIST_INSERTED|EVLIST_ACTIVE))) {
 		if (ev->ev_events & (EV_READ|EV_WRITE))
@@ -2012,7 +2102,7 @@ event_add_internal(struct event *ev, const struct timeval *tv,
 	}
 
 	/* if we are not in the right thread, we need to wake up the loop */
-	if (res != -1 && notify && !EVBASE_IN_THREAD(base))
+	if (res != -1 && notify && EVBASE_NEED_NOTIFY(base))
 		evthread_notify_base(base);
 
 	_event_debug_note_add(ev);
@@ -2025,6 +2115,11 @@ event_del(struct event *ev)
 {
 	int res;
 
+	if (EVUTIL_FAILURE_CHECK(!ev->ev_base)) {
+		event_warnx("%s: event has no event_base set.", __func__);
+		return -1;
+	}
+
 	EVBASE_ACQUIRE_LOCK(ev->ev_base, th_base_lock);
 
 	res = event_del_internal(ev);
@@ -2040,10 +2135,9 @@ event_del_internal(struct event *ev)
 {
 	struct event_base *base;
 	int res = 0, notify = 0;
-	int need_cur_lock;
 
-	event_debug(("event_del: %p, callback %p",
-		 ev, ev->ev_callback));
+	event_debug(("event_del: %p (fd %d), callback %p",
+		ev, (int)ev->ev_fd, ev->ev_callback));
 
 	/* An event without a base has not been added */
 	if (ev->ev_base == NULL)
@@ -2057,9 +2151,12 @@ event_del_internal(struct event *ev)
 	 * when this function returns, it will be safe to free the
 	 * user-supplied argument. */
 	base = ev->ev_base;
-	need_cur_lock = (base->current_event == ev);
-	if (need_cur_lock)
-		EVBASE_ACQUIRE_LOCK(base, current_event_lock);
+#ifndef _EVENT_DISABLE_THREAD_SUPPORT
+	if (base->current_event == ev && !EVBASE_IN_THREAD(base)) {
+		++base->current_event_waiters;
+		EVTHREAD_COND_WAIT(base->current_event_cond, base->th_base_lock);
+	}
+#endif
 
 	EVUTIL_ASSERT(!(ev->ev_flags & ~EVLIST_ALL));
 
@@ -2099,12 +2196,9 @@ event_del_internal(struct event *ev)
 	}
 
 	/* if we are not in the right thread, we need to wake up the loop */
-	if (res != -1 && notify && !EVBASE_IN_THREAD(base))
+	if (res != -1 && notify && EVBASE_NEED_NOTIFY(base))
 		evthread_notify_base(base);
 
-	if (need_cur_lock)
-		EVBASE_RELEASE_LOCK(base, current_event_lock);
-
 	_event_debug_note_del(ev);
 
 	return (res);
@@ -2113,6 +2207,11 @@ event_del_internal(struct event *ev)
 void
 event_active(struct event *ev, int res, short ncalls)
 {
+	if (EVUTIL_FAILURE_CHECK(!ev->ev_base)) {
+		event_warnx("%s: event has no event_base set.", __func__);
+		return;
+	}
+
 	EVBASE_ACQUIRE_LOCK(ev->ev_base, th_base_lock);
 
 	_event_debug_assert_is_setup(ev);
@@ -2128,6 +2227,10 @@ event_active_nolock(struct event *ev, int res, short ncalls)
 {
 	struct event_base *base;
 
+	event_debug(("event_active: %p (fd %d), res %d, callback %p",
+		ev, (int)ev->ev_fd, (int)res, ev->ev_callback));
+
+
 	/* We get different kinds of events, add them together */
 	if (ev->ev_flags & EVLIST_ACTIVE) {
 		ev->ev_res |= res;
@@ -2141,6 +2244,12 @@ event_active_nolock(struct event *ev, int res, short ncalls)
 	ev->ev_res = res;
 
 	if (ev->ev_events & EV_SIGNAL) {
+#ifndef _EVENT_DISABLE_THREAD_SUPPORT
+		if (base->current_event == ev && !EVBASE_IN_THREAD(base)) {
+			++base->current_event_waiters;
+			EVTHREAD_COND_WAIT(base->current_event_cond, base->th_base_lock);
+		}
+#endif
 		ev->ev_ncalls = ncalls;
 		ev->ev_pncalls = NULL;
 	}
@@ -2192,8 +2301,6 @@ event_deferred_cb_schedule(struct deferred_cb_queue *queue,
 		cb->queued = 1;
 		TAILQ_INSERT_TAIL(&queue->deferred_cb_list, cb, cb_next);
 		++queue->active_count;
-		/* XXXX Can we get away with doing this only when adding
-		 * the first active deferred_cb to the queue? */
 		if (queue->notify_fn)
 			queue->notify_fn(queue, queue->notify_arg);
 	}
@@ -2539,18 +2646,23 @@ evthread_notify_drain_eventfd(evutil_socket_t fd, short what, void *arg)
 {
 	ev_uint64_t msg;
 	ev_ssize_t r;
+	struct event_base *base = arg;
 
 	r = read(fd, (void*) &msg, sizeof(msg));
 	if (r<0 && errno != EAGAIN) {
 		event_sock_warn(fd, "Error reading from eventfd");
 	}
+	EVBASE_ACQUIRE_LOCK(base, th_base_lock);
+	base->is_notify_pending = 0;
+	EVBASE_RELEASE_LOCK(base, th_base_lock);
 }
 #endif
 
 static void
 evthread_notify_drain_default(evutil_socket_t fd, short what, void *arg)
 {
-	unsigned char buf[128];
+	unsigned char buf[1024];
+	struct event_base *base = arg;
 #ifdef WIN32
 	while (recv(fd, (char*)buf, sizeof(buf), 0) > 0)
 		;
@@ -2558,6 +2670,10 @@ evthread_notify_drain_default(evutil_socket_t fd, short what, void *arg)
 	while (read(fd, (char*)buf, sizeof(buf)) > 0)
 		;
 #endif
+
+	EVBASE_ACQUIRE_LOCK(base, th_base_lock);
+	base->is_notify_pending = 0;
+	EVBASE_RELEASE_LOCK(base, th_base_lock);
 }
 
 int
@@ -2578,16 +2694,15 @@ evthread_make_base_notifiable(struct event_base *base)
 	if (base->th_notify_fd[0] >= 0) {
 		notify = evthread_notify_base_eventfd;
 		cb = evthread_notify_drain_eventfd;
-	} else
+	}
 #endif
 #if defined(_EVENT_HAVE_PIPE)
-	{
+	if (base->th_notify_fd[0] < 0) {
 		if ((base->evsel->features & EV_FEATURE_FDS)) {
 			if (pipe(base->th_notify_fd) < 0)
 				event_warn("%s: pipe", __func__);
 		}
 	}
-	if (base->th_notify_fd[0] < 0)
 #endif
 
 #ifdef WIN32
@@ -2595,7 +2710,7 @@ evthread_make_base_notifiable(struct event_base *base)
 #else
 #define LOCAL_SOCKETPAIR_AF AF_UNIX
 #endif
-	{
+	if (base->th_notify_fd[0] < 0) {
 		if (evutil_socketpair(LOCAL_SOCKETPAIR_AF, SOCK_STREAM, 0,
 			base->th_notify_fd) == -1) {
 			event_sock_warn(-1, "%s: socketpair", __func__);
@@ -2608,10 +2723,15 @@ evthread_make_base_notifiable(struct event_base *base)
 	base->th_notify_fn = notify;
 
 	/*
-	  This can't be right, can it?  We want writes to this socket to
-	  just succeed.
-	  evutil_make_socket_nonblocking(base->th_notify_fd[1]);
+	  Making the second socket nonblocking is a bit subtle, given that we
+	  ignore any EAGAIN returns when writing to it, and you don't usally
+	  do that for a nonblocking socket. But if the kernel gives us EAGAIN,
+	  then there's no need to add any more data to the buffer, since
+	  the main thread is already either about to wake up and drain it,
+	  or woken up and in the process of draining it.
 	*/
+	if (base->th_notify_fd[1] > 0)
+		evutil_make_socket_nonblocking(base->th_notify_fd[1]);
 
 	/* prepare an event that we can use for wakeup */
 	event_assign(&base->th_notify, base, base->th_notify_fd[0],
@@ -2619,6 +2739,7 @@ evthread_make_base_notifiable(struct event_base *base)
 
 	/* we need to mark this as internal event */
 	base->th_notify.ev_flags |= EVLIST_INTERNAL;
+	event_priority_set(&base->th_notify, 0);
 
 	return event_add(&base->th_notify, NULL);
 }
@@ -2653,3 +2774,22 @@ event_base_dump_events(struct event_base *base, FILE *output)
 		}
 	}
 }
+
+void
+event_base_add_virtual(struct event_base *base)
+{
+	EVBASE_ACQUIRE_LOCK(base, th_base_lock);
+	base->virtual_event_count++;
+	EVBASE_RELEASE_LOCK(base, th_base_lock);
+}
+
+void
+event_base_del_virtual(struct event_base *base)
+{
+	EVBASE_ACQUIRE_LOCK(base, th_base_lock);
+	EVUTIL_ASSERT(base->virtual_event_count > 0);
+	base->virtual_event_count--;
+	if (base->virtual_event_count == 0 && EVBASE_NEED_NOTIFY(base))
+		evthread_notify_base(base);
+	EVBASE_RELEASE_LOCK(base, th_base_lock);
+}
