@@ -37,7 +37,7 @@
 extern "C" {
 #endif
 
-#include <event-config.h>
+#include <event2/event-config.h>
 #ifdef _EVENT_HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif
@@ -191,7 +191,7 @@ enum event_base_config_flag {
 	/** Instead of checking the current time every time the event loop is
 	    ready to run timeout callbacks, check after each timeout callback.
 	 */
-	EVENT_BASE_FLAG_NO_CACHE_TIME = 0x08,
+	EVENT_BASE_FLAG_NO_CACHE_TIME = 0x08
 };
 
 /**
@@ -226,6 +226,16 @@ int event_config_require_features(struct event_config *cfg, int feature);
  * will be initialized, and how they'll work. */
 int event_config_set_flag(struct event_config *cfg, int flag);
 
+/**
+ * Records a hint for the number of CPUs in the system. This is used for
+ * tuning thread pools, etc, for optimal performance.
+ *
+ * @param cfg the event configuration object
+ * @param cpus the number of cpus
+ * @return 0 on success, -1 on failure.
+ */
+int event_config_set_num_cpus_hint(struct event_config *cfg, int cpus);
+
 /**
   Initialize the event API.
 
@@ -605,9 +615,14 @@ void event_get_assignment(const struct event *event, struct event_base **base_ou
    Return the size of struct event that the Libevent library was compiled
    with.
 
-   This will be the same as sizeof(struct event) if you're running with the
-   same version of Libevent that your application was built with, but
+   This will be NO GREATER than sizeof(struct event) if you're running with
+   the same version of Libevent that your application was built with, but
    otherwise might not.
+
+   Note that it might be SMALLER than sizeof(struct event) if some future
+   version of Libevent adds extra padding to the end of struct event.
+   We might do this to help ensure ABI-compatibility between different
+   versions of Libevent.
  */
 size_t event_get_struct_event_size(void);
 
